@@ -23,7 +23,7 @@ $evh->request_handler(sub {
     my $r = shift;
     my $n = $counter++;
     my $env = $r->env;
-    $r->send_response("200 OK", [
+    $r->send_response(200, [
         'Content-Type' => 'text/plain',
         'Connection' => 'close',
     ], \"Hello customer number $n\n");
@@ -33,4 +33,4 @@ my $t = EV::timer 1, 1, sub {
     print "served $counter\n";
 };
 
-EV::loop;
+EV::run;
