@@ -345,9 +345,12 @@ try_tls_conn_read(EV_P_ ev_io *w, int revents)
 
         if (remaining > 0) {
             /* Leftover bytes are the start of the TLS ClientHello.
-             * Copy them into rawbuf and fall through to the handshake. */
-            memcpy(rawbuf, SvPVX(c->rbuf) + ret, remaining);
-            nread = (ssize_t)remaining;
+             * Save in tls_rbuf (heap) to avoid overflowing stack rawbuf
+             * when the PROXY header spans multiple reads. */
+            Newx(c->tls_rbuf, remaining, uint8_t);
+            memcpy(c->tls_rbuf, SvPVX(c->rbuf) + ret, remaining);
+            c->tls_rbuf_len = remaining;
+            nread = 0;
         }
         SvREFCNT_dec(c->rbuf);
         c->rbuf = NULL;
@@ -604,6 +607,33 @@ tls_parse:
             stop_read_timer(c);
         }
     }
+    else if (c->receiving == RECEIVE_CHUNKED) {
+        int ret = try_parse_chunked(c);
+        if (ret == -1) {
+            respond_with_server_error(c, "Malformed chunked encoding\n", 0, 400);
+            change_receiving_state(c, RECEIVE_SHUTDOWN);
+            stop_read_watcher(c);
+            stop_read_timer(c);
+        }
+        else if (ret == 0) {
+            sched_request_callback(c);
+            change_receiving_state(c, RECEIVE_SHUTDOWN);
+            stop_read_watcher(c);
+            stop_read_timer(c);
+        }
+        /* ret == 1: need more data, watcher stays active */
+    }
+    else if (c->receiving == RECEIVE_STREAMING) {
+        c->received_cl += got_n;
+        if (c->poll_read_cb) {
+            call_poll_callback(c, 0);
+        }
+        if (c->expected_cl > 0 && c->received_cl >= c->expected_cl) {
+            change_receiving_state(c, RECEIVE_SHUTDOWN);
+            stop_read_watcher(c);
+            stop_read_timer(c);
+        }
+    }
 
 tls_read_cleanup:
     SvREFCNT_dec(c->self);
@@ -674,8 +704,8 @@ try_tls_conn_write(EV_P_ ev_io *w, int revents)
                 if (ret != 0) {
                     ptls_buffer_dispose(&encbuf);
                     trouble("ptls_send error fd=%d ret=%d\n", c->fd, ret);
-                    /* Free remaining SVs in this matrix */
-                    for (; i < m->count; i++) {
+                    /* Free all SVs in this matrix */
+                    for (i = 0; i < m->count; i++) {
                         if (m->sv[i]) SvREFCNT_dec(m->sv[i]);
                     }
                     IOMATRIX_FREE(m);
@@ -685,7 +715,7 @@ try_tls_conn_write(EV_P_ ev_io *w, int revents)
                     if (tls_wbuf_append(&c->tls_wbuf, encbuf.base, encbuf.off) != 0) {
                         ptls_buffer_dispose(&encbuf);
                         trouble("TLS wbuf alloc failed fd=%d\n", c->fd);
-                        for (; i < m->count; i++) {
+                        for (i = 0; i < m->count; i++) {
                             if (m->sv[i]) SvREFCNT_dec(m->sv[i]);
                         }
                         IOMATRIX_FREE(m);
