@@ -29,7 +29,7 @@ use constant DEATH_TIMER => 5.0; # seconds
 use constant DEATH_TIMER_INCR => 2.0; # seconds
 use constant DEFAULT_HOST => 'localhost';
 use constant DEFAULT_PORT => 5000;
-use constant MAX_PRE_FORK => 1000; # croak if pre_fork exceeds this
+use constant MAX_PRE_FORK => $ENV{FEERSUM_MAX_PRE_FORK} || 1000;
 
 our $INSTANCE;
 sub new { ## no critic (RequireArgUnpacking)
