@@ -27,6 +27,8 @@ sub import {
     *{$pkg.'::blessed'} = \&Scalar::Util::blessed;
     *{$pkg.'::get_listen_socket'} = \&get_listen_socket;
     *{$pkg.'::simple_client'} = \&simple_client;
+    *{$pkg.'::build_proxy_v1'} = \&build_proxy_v1;
+    *{$pkg.'::build_proxy_v2'} = \&build_proxy_v2;
 
     return 1;
 }
@@ -205,4 +207,74 @@ sub simple_client ($$;@) {
     return $strong_h;
 }
 
+# Build PROXY protocol v1 header (text format)
+# Usage: build_proxy_v1('TCP4', '192.0.2.1', '192.0.2.2', 12345, 80)
+#        build_proxy_v1('UNKNOWN')  # for health checks
+sub build_proxy_v1 {
+    my ($proto, $src_ip, $dst_ip, $src_port, $dst_port) = @_;
+
+    if ($proto eq 'UNKNOWN') {
+        return "PROXY UNKNOWN\r\n";
+    }
+
+    return "PROXY $proto $src_ip $dst_ip $src_port $dst_port\r\n";
+}
+
+# Build PROXY protocol v2 header (binary format)
+# $cmd: 'LOCAL' or 'PROXY'
+# $family: 'UNSPEC', 'INET' (IPv4), or 'INET6' (IPv6)
+# For PROXY command with INET/INET6, provide addresses
+sub build_proxy_v2 {
+    my ($cmd, $family, $src_ip, $dst_ip, $src_port, $dst_port, $tlvs) = @_;
+
+    # v2 signature
+    my $sig = "\x0D\x0A\x0D\x0A\x00\x0D\x0A\x51\x55\x49\x54\x0A";
+
+    # version (2) and command
+    my $ver_cmd;
+    if ($cmd eq 'LOCAL') {
+        $ver_cmd = 0x20 | 0x00;  # version 2, LOCAL command
+    } elsif ($cmd eq 'PROXY') {
+        $ver_cmd = 0x20 | 0x01;  # version 2, PROXY command
+    } else {
+        croak "Unknown command: $cmd";
+    }
+
+    # family and protocol
+    my ($fam_proto, $addr_data);
+    if ($family eq 'UNSPEC') {
+        $fam_proto = 0x00;
+        $addr_data = '';
+    } elsif ($family eq 'INET') {
+        $fam_proto = 0x11;  # AF_INET + STREAM
+        # Pack IPv4 addresses
+        my $src_packed = Socket::inet_aton($src_ip) or croak "Invalid src IP: $src_ip";
+        my $dst_packed = Socket::inet_aton($dst_ip) or croak "Invalid dst IP: $dst_ip";
+        $addr_data = $src_packed . $dst_packed . pack('nn', $src_port, $dst_port);
+    } elsif ($family eq 'INET6') {
+        $fam_proto = 0x21;  # AF_INET6 + STREAM
+        # Pack IPv6 addresses
+        my $src_packed = Socket::inet_pton(Socket::AF_INET6(), $src_ip)
+            or croak "Invalid src IPv6: $src_ip";
+        my $dst_packed = Socket::inet_pton(Socket::AF_INET6(), $dst_ip)
+            or croak "Invalid dst IPv6: $dst_ip";
+        $addr_data = $src_packed . $dst_packed . pack('nn', $src_port, $dst_port);
+    } else {
+        croak "Unknown family: $family";
+    }
+
+    # Append TLVs if provided (arrayref of [type, value] pairs)
+    my $tlv_data = '';
+    if ($tlvs && ref($tlvs) eq 'ARRAY') {
+        for my $tlv (@$tlvs) {
+            my ($type, $value) = @$tlv;
+            $tlv_data .= chr($type) . pack('n', length($value)) . $value;
+        }
+    }
+
+    my $addr_len = length($addr_data) + length($tlv_data);
+
+    return $sig . chr($ver_cmd) . chr($fam_proto) . pack('n', $addr_len) . $addr_data . $tlv_data;
+}
+
 1;
